#ifndef HARKIO_TYPEDFILENAME_HPP_
#define HARKIO_TYPEDFILENAME_HPP_

#include <boost/operators.hpp>
#include <glibmm/ustring.h>
#include <harkio2/FileType.hpp>

/** @file TypedFilename.hpp
 * @brief TypedFilename class definition.
 */

namespace harkio2
{

/** @brief This class represents filename and filetype.
 */
class TypedFilename :
    private boost::equality_comparable<TypedFilename> // for != operator
{
public:
    /** @brief Constructs blank data structure.
     */
    TypedFilename()
        : name(), type(harkio2::FileType::FLOAT)
    {}

    /** @brief Constructs filename with 'type'.
     * @param name The filename.
     * @param type The type of the file.
     */
    TypedFilename(const Glib::ustring& name, harkio2::FileType::Kind type)
        : name(name), type(type)
    {}

    /** @brief Returns the filename.
     * @return The filename.
     */
    const Glib::ustring& GetName() const
    {
        return this->name;
    }

    /** @brief Returns the file type.
     * @return The file type.
     */
    harkio2::FileType::Kind GetType() const
    {
        return this->type;
    }

    /** @brief Returns true if specified object equal to this filename.
     * @return true if specified object equal to this filename.
     */
    bool operator ==(const TypedFilename& rhs) const
    {
        return this->name == rhs.name && this->type == rhs.type;
    }

private:
    /// @brief filename.
    Glib::ustring name;

    /// @brief file type.
    harkio2::FileType::Kind type;
};

}

#endif // HARKIO_TYPEDFILENAME_HPP_

