#ifndef HARKIO_HARK_HGTF_SM_HPP_
#define HARKIO_HARK_HGTF_SM_HPP_

#include <string>
#include <vector>
#include <complex>
#include <glibmm/ustring.h>
#include <harkio2/Version.hpp>
#include <harkio2/Vector3.hpp>
#include <harkio2/defineitem.hpp>
#include <harkio2/InvalidHarkFile.hpp>
#include <harkio2/hark/HarkFileWriter.hpp>

/** @file SM.hpp
 * @brief SM class and subclasses definitions.
 */

namespace harkio2
{
namespace hark
{
namespace hgtf
{

/** @brief Abstract SM data set.
 * This class represents a set of separation matrices.
 */
class SM : public harkio2::hark::DataSet
{
public:
    /** @brief Returns the type identifier.
     * @return The type identifier
     */
    virtual std::string GetId() const { return "hgtf::SM"; }

    /** @brief Returns the version.
     * Returns the version which this instance actually represents.
     * @return The version number.
     */
    virtual harkio2::Version GetVersion() const = 0;
};

/** @brief SM data set version 1.0.
 */
class SM1_0 : public SM
{
public:
    /** @brief Returns the version.
     * @return The version which this class represents.
     */
    virtual harkio2::Version GetVersion() const { return harkio2::Version(1, 0); }

    /** @brief Separation matrix type.
     * This type represents a separation matrix.
     */
    typedef std::vector<std::vector<std::complex<float> > > SMType;

    DEFINE_ITEM(nSource, unsigned int, NSource, unsigned int)
    DEFINE_ITEM(sourcePositions, std::vector<harkio2::Vector3<float> >,
            SourcePositions, const std::vector<harkio2::Vector3<float> >&)
    DEFINE_ITEM(nFIndex, unsigned int, NFIndex, unsigned int)
    DEFINE_ITEM(fIndexToBinIndexTable, std::vector<unsigned int>, FIndexToBinIndexTable, const std::vector<unsigned int>&)
    DEFINE_ITEM(sms, std::vector<SMType>, SMs, const std::vector<SMType>&)

    /// @brief Implementation of Writable interface.
    void Write(HarkFileWriter& writer) throw (harkio2::InvalidHarkFile)
    {
        writer.Write(*this);
    }

};

}
}
}

#endif // HARKIO_HARK_HGTF_SM_HPP_

