#include <iostream>
#include <string>
#include <fstream>
#include <cstring>
#include <harkio2/harkio.hpp>

using namespace std;
using namespace boost;
using namespace harkio2;

int main(int argc, char** argv)
{
    if (argc != 3)
    {
        cout << "Usage: " << argv[0] << " hark-config|hark <file name>" << endl;
        return 0;
    }

    if (strcmp(argv[1], "hark-config") == 0)
    {
        try
        {
            shared_ptr<harkconfig::HarkConfigFileReader> reader(
                    new harkconfig::HarkConfigFileXMLReader(argv[2]));
            shared_ptr<harkconfig::TSPListFile2_0> tsp =
                dynamic_pointer_cast<harkconfig::TSPListFile2_0>(
                        reader->Read());

            if (tsp == 0)
            {
                cout << "this is not TSPListFile 2.0" << endl;
                return -1;
            }

            cout << "SynchronousAverage: " << tsp->GetSynchronousAverage() << endl;
            cout << "OriginalImpulseFile: " << tsp->GetOriginalImpulseFile().GetName() << endl;

            string newTspFilename = string("newtsp_") + string(argv[2]);
            cout << "writing TSPListFile2_0 data structure to " << newTspFilename << endl;
            ofstream newTspFile(newTspFilename.c_str());
            shared_ptr<harkconfig::HarkConfigFileWriter> writer(
                    new harkconfig::HarkConfigFileXMLWriter(newTspFile));
            writer->Write(tsp);
        }
        catch (const harkio2::AnalyzeError& e)
        {
            cout << e.line() << ": " << e.what() << endl;
            return -1;
        }
    }
    else if (strcmp(argv[1], "hark") == 0)
    {
        try
        {
            ifstream harkFileData(argv[2]);
            shared_ptr<hark::HarkFileReader> reader(
                    new hark::HarkFileBinaryReader(harkFileData));
            shared_ptr<hark::HarkFile> hark = reader->Read();

            shared_ptr<hark::hgtf::Fmt1_0> fmt10 =
                dynamic_pointer_cast<hark::hgtf::Fmt1_0>(
                        hark->GetDataSet("hgtf::Fmt"));

            if (fmt10 == 0)
            {
                cout << "this is not include Fmt dataset version 1.0" << endl;
                return -1;
            }

            cout << "the number of mic: " << fmt10->GetNMic() << endl;
            cout << "the number of FFT points: " << fmt10->GetNFFT() << endl;

            ofstream newHarkFileData((string(argv[2]) + ".new").c_str());
            shared_ptr<hark::HarkFileWriter> writer(
                    new hark::HarkFileBinaryWriter(newHarkFileData));
            writer->Write(hark);
        }
        catch (const harkio2::AnalyzeError& e)
        {
            cout << e.what() << endl;
            return -1;
        }

    }
    else if (strcmp(argv[1], "test") == 0)
    {
        {
            std::ifstream ifs(argv[2]);
            harkio2::hark::HarkFileBinaryReader reader(ifs);
            boost::shared_ptr<harkio2::hark::HarkFile> hark = reader.Read();
        }

        {
            harkio2::harkconfig::HarkConfigFileXMLReader reader("sample_tsp20.xml");
            boost::shared_ptr<harkio2::harkconfig::HarkConfigFile> harkConfig = reader.Read();

            std::ofstream ofs("new_sample_tsp20.xml");
            harkio2::harkconfig::HarkConfigFileXMLWriter writer(ofs);
            writer.Write(harkConfig);
        }
    }

    return 0;
}

